#include <ESP8266WiFi.h>
#include <WiFiUDP.h>
#include <ArduinoJson.h>

// Wifi config
char ssid[] = "";     // Fill in your network SSID
char password[] = ""; // Fill in your network password 

// Multicast ip address + port definition
IPAddress ipMulti(224, 0, 0, 50);
unsigned int portMulti = 9898; // local port to listen on

// Loxone IP address + port definition
IPAddress loxoneIP(255, 255, 255, 255);
unsigned int loxonePort = 56666;

// Packet, loxone and JSON buffers
char packetBuffer[UDP_TX_PACKET_MAX_SIZE];
char loxoneBuffer[UDP_TX_PACKET_MAX_SIZE];

// Led states
#define LED_IDLE HIGH
#define LED_BUSY LOW

WiFiUDP Udp;

void setup() {
  Serial.begin(115200);
  delay(10);

  // Handle LED
  pinMode(BUILTIN_LED, OUTPUT);
  digitalWrite(BUILTIN_LED, LED_BUSY);
  
  Serial.println();
  Serial.println();
  Serial.print("Connecting to ");
  Serial.println(ssid);

  // Connect to Wifi
  WiFi.begin(ssid, password);
  while (WiFi.status() != WL_CONNECTED) {
    delay(500);
    Serial.print(".");
  }
  Serial.println("");
  Serial.println("WiFi connected");

  // Print the SSID of the network you're attached to:
  Serial.print("SSID: "); Serial.println(WiFi.SSID());

  // Print your WiFi shield's IP address:
  IPAddress ip = WiFi.localIP();
  Serial.print("IP Address: "); Serial.println(ip);

  // Print the received signal strength:
  long rssi = WiFi.RSSI();
  Serial.print("Signal strength (RSSI):"); Serial.print(rssi); Serial.println(" dBm");

  // Start UDP multicast
  Serial.print("UDP Multicast listener starting at: ");
  Serial.print(ipMulti); Serial.print(":"); Serial.println(portMulti);
  Udp.beginMulticast(WiFi.localIP(), ipMulti, portMulti);

  // Handle LED
  digitalWrite(BUILTIN_LED, LED_IDLE);
}
 
void loop() {
  // If there is data available, read the packet
  int packetSize = Udp.parsePacket();
  if (packetSize) {
    // Handle LED
    digitalWrite(BUILTIN_LED, LED_BUSY);
    
    // Prepare packet buffer
    for (int i=0; i < UDP_TX_PACKET_MAX_SIZE; i++)
      packetBuffer[i] = 0;
    
    // Read the packet into packetBufffer
    Udp.read(packetBuffer, packetSize);
    Serial.print("Received Xiaomi packet ("); Serial.print(strlen(packetBuffer)); Serial.println(" bytes):");
    Serial.println(packetBuffer);

    // Parse Xiaomi packet
    StaticJsonBuffer<UDP_TX_PACKET_MAX_SIZE> jsonBuffer;
    JsonObject& root = jsonBuffer.parseObject(packetBuffer);
    if (!root.success()) {
      Serial.println("JSON parseObject() failed for packetBuffer");
      return;
    }

    // Check model, sid and data
    if (!root.containsKey("model") || !root.containsKey("sid") || !root.containsKey("data")) {
      Serial.println("Packet missing model, sid or data");
      return;
    }
    String model = root["model"];
    String sid = root["sid"];
    String data = root["data"];
    
    // Parse data JSON
    StaticJsonBuffer<UDP_TX_PACKET_MAX_SIZE> dataBuffer;
    JsonObject& dataJSON = dataBuffer.parseObject(data);
    if (!dataJSON.success()) {
      Serial.println("JSON parseObject() failed for data");
      return;
    }

    // Loop over the data
    for (JsonObject::iterator it=dataJSON.begin(); it!=dataJSON.end(); ++it) {
      // Prepare packet
      String key = it->key;
      String value = it->value;
      String loxonePacket = model + " " + sid + " " + key + " " + value;

      loxonePacket.toCharArray(loxoneBuffer, UDP_TX_PACKET_MAX_SIZE);

      // Send packet to Loxone
      Serial.print("Sending loxone packet ("); Serial.print(strlen(loxoneBuffer)); Serial.println(" bytes):");
      Serial.println(loxonePacket);
      Udp.beginPacket(loxoneIP, loxonePort);
      Udp.write(loxoneBuffer);
      Udp.endPacket();
    }

    // Handle LED
    digitalWrite(BUILTIN_LED, LED_IDLE);
  }
}
